@@ -1,5 +1,5 @@
 if [[ $TRAVIS_PYTHON_VERSION == 3* ]]; then
-    cd pysal; python -m 'nose';
+    nosetests --with-coverage --cover-package=pysal --exclude-dir=pysal/contrib
 else
     nosetests --with-coverage --cover-package=pysal; 
 fi
