@@ -10,6 +10,7 @@ import doctest
 import math
 from warnings import warn
 from sphere import arcdist
+import numpy as np
 
 __all__ = ['Point', 'LineSegment', 'Line', 'Ray', 'Chain', 'Polygon',
            'Rectangle', 'asShape']
@@ -20,6 +21,8 @@ def asShape(obj):
     Returns a pysal shape object from obj.
     obj must support the __geo_interface__.
     """
+    if isinstance(obj, (Point, LineSegment, Line, Ray, Chain, Polygon)):
+        return obj
     if hasattr(obj, '__geo_interface__'):
         geo = obj.__geo_interface__
     else:
@@ -35,8 +38,15 @@ def asShape(obj):
         raise NotImplementedError(
             "%s is not supported at this time." % geo_type)
 
+class Geometry(object):
+    """
+    A base class to help implement is_geometry and make geometric types
+    extendable.
+    """
+    def __init__(self):
+        pass
 
-class Point(object):
+class Point(Geometry):
     """
     Geometric class for point objects.
 
@@ -319,7 +329,7 @@ class Point(object):
         >>> Point((0,1))
         (0.0, 1.0)
         """
-        return self.__loc.__repr__()
+        return str(self)
 
     def __str__(self):
         """
@@ -340,9 +350,10 @@ class Point(object):
         '(1.0, 3.0)'
         """
         return str(self.__loc)
+        return "POINT ({} {})".format(*self.__loc)
 
 
-class LineSegment(object):
+class LineSegment(Geometry):
     """
     Geometric representation of line segment objects.
 
@@ -394,6 +405,8 @@ class LineSegment(object):
 
     def __str__(self):
         return "LineSegment(" + str(self._p1) + ", " + str(self._p2) + ")"
+        return "LINESTRING ({} {}, {} {})".format(self._p1[0], self._p1[1], 
+                                                  self._p2[0], self._p2[1])
 
     def __eq__(self, other):
         """
@@ -771,7 +784,7 @@ class LineSegment(object):
         return self._line
 
 
-class VerticalLine:
+class VerticalLine(Geometry):
     """
     Geometric representation of verticle line objects.
 
@@ -848,7 +861,7 @@ class VerticalLine:
         return float('nan')
 
 
-class Line:
+class Line(Geometry):
     """
     Geometric representation of line objects.
 
@@ -977,7 +990,7 @@ class Ray:
         self.p = second_p
 
 
-class Chain(object):
+class Chain(Geometry):
     """
     Geometric representation of a chain, also known as a polyline.
 
@@ -1013,7 +1026,7 @@ class Chain(object):
         else:
             self._vertices = [vertices]
         self._reset_props()
-
+    
     @classmethod
     def __from_geo_interface__(cls, geo):
         if geo['type'].lower() == 'linestring':
@@ -1175,7 +1188,7 @@ class Chain(object):
         return [[LineSegment(a, b) for (a, b) in zip(part[:-1], part[1:])] for part in self._vertices]
 
 
-class Ring(object):
+class Ring(Geometry):
     """
     Geometric representation of a Linear Ring
 
@@ -1285,9 +1298,10 @@ class Ring(object):
 
             A = 0.0
             for i in xrange(N - 1):
-                A += (x[i] * y[i + 1] - x[i + 1] * y[i])
-            A = A / 2.0
-            self._area = A
+                A += (x[i] + x[i + 1]) * \
+                    (y[i] - y[i + 1])
+            A = A * 0.5
+            self._area = -A
         return self._area
 
     @property
@@ -1380,7 +1394,7 @@ class Ring(object):
 
 
 
-class Polygon(object):
+class Polygon(Geometry):
     """
     Geometric representation of polygon objects.
 
@@ -1751,7 +1765,7 @@ class Polygon(object):
 
 
 
-class Rectangle:
+class Rectangle(Geometry):
     """
     Geometric representation of rectangle objects.
 
