@@ -149,7 +149,10 @@ def _unpackDict2(d, structure, fileObj):
     for name, dtype, order in structure:
         dtype, n = dtype
         result = array.array(dtype)
-        result.fromstring(fileObj.read(result.itemsize * n))
+        try:
+            result.frombytes(fileObj.read(result.itemsize * n))
+        except AttributeError:
+            result.fromstring(fileObj.read(result.itemsize * n))
         if order != SYS_BYTE_ORDER:
             result.byteswap()
         d[name] = result.tolist()
@@ -733,6 +736,7 @@ class Polygon(PolyLine):
 
 
 class MultiPoint:
+    String_Type = 'MULTIPOINT'
     def __init__(self):
         raise NotImplementedError("No MultiPoint Support at this time.")
 
@@ -742,31 +746,37 @@ class PolygonZ(PolyLineZ):
 
 
 class MultiPointZ:
+    String_Type = 'MULTIPOINTZ'
     def __init__(self):
         raise NotImplementedError("No MultiPointZ Support at this time.")
 
 
 class PointM:
+    String_Type = 'POINTM'
     def __init__(self):
         raise NotImplementedError("No PointM Support at this time.")
 
 
 class PolyLineM:
+    String_Type = 'ARCM'
     def __init__(self):
         raise NotImplementedError("No PolyLineM Support at this time.")
 
 
 class PolygonM:
+    String_Type = 'POLYGONM'
     def __init__(self):
         raise NotImplementedError("No PolygonM Support at this time.")
 
 
 class MultiPointM:
+    String_Type = 'MULTIPOINTM'
     def __init__(self):
         raise NotImplementedError("No MultiPointM Support at this time.")
 
 
 class MultiPatch:
+    String_Type = 'MULTIPATCH'
     def __init__(self):
         raise NotImplementedError("No MultiPatch Support at this time.")
 
