@@ -4,8 +4,9 @@ Moran's I Spatial Autocorrelation Statistics
 """
 __author__ = "Sergio J. Rey <srey@asu.edu>, \
         Dani Arribas-Bel <daniel.arribas.bel@gmail.com>"
-from pysal.weights.spatial_lag import lag_spatial as slag
-from pysal.esda.smoothing import assuncao_rate
+from ..weights.spatial_lag import lag_spatial as slag
+from .smoothing import assuncao_rate
+from .tabular import _univariate_handler, _bivariate_handler
 import scipy.stats as stats
 import numpy as np
 
@@ -15,7 +16,7 @@ __all__ = ["Moran", "Moran_Local", "Moran_BV", "Moran_BV_matrix",
 PERMUTATIONS = 999
 
 
-class Moran:
+class Moran(object):
     """Moran's I Global Autocorrelation Statistic
 
     Parameters
@@ -101,7 +102,7 @@ class Moran:
     >>> w = pysal.open(pysal.examples.get_path("stl.gal")).read()
     >>> f = pysal.open(pysal.examples.get_path("stl_hom.txt"))
     >>> y = np.array(f.by_col['HR8893'])
-    >>> mi = Moran(y,  w)
+    >>> mi = pysal.Moran(y,  w)
     >>> "%7.5f" % mi.I
     '0.24366'
     >>> mi.EI
@@ -113,6 +114,7 @@ class Moran:
 
     >>> w = pysal.open(pysal.examples.get_path("sids2.gal")).read()
     >>> f = pysal.open(pysal.examples.get_path("sids2.dbf"))
+    >>> import numpy as np
     >>> SIDR = np.array(f.by_col("SIDR74"))
     >>> mi = pysal.Moran(SIDR,  w)
     >>> "%6.4f" % mi.I
@@ -128,9 +130,19 @@ class Moran:
     >>> mi_1.p_norm
     5.7916539074498452e-05
 
+    Example from http://www.lpc.uottawa.ca/publications/moransi/moran.htm
+
+    >>> w = pysal.lat2W(3, 3)
+    >>> y = np.arange(1, 10)
+    >>> mi = pysal.Moran(y, w, transformation='B')
+    >>> mi.VI_rand
+    0.059687500000000004
+    >>> mi.VI_norm
+    0.053125000000000006
     """
     def __init__(self, y, w, transformation="r", permutations=PERMUTATIONS,
                  two_tailed=True):
+        y = np.asarray(y).flatten()
         self.y = y
         w.transform = transformation
         self.w = w
@@ -177,30 +189,82 @@ class Moran:
         self.z2ss = (z * z).sum()
         self.EI = -1. / (self.n - 1)
         n = self.n
+        n2 = n * n
         s1 = self.w.s1
         s0 = self.w.s0
         s2 = self.w.s2
         s02 = s0 * s0
-        v_num = n * n * s1 - n * s2 + 3 * s0 * s0
-        v_den = (n - 1) * (n + 1) * s0 * s0
+        v_num = n2  * s1 - n * s2 + 3 * s02
+        v_den = (n - 1) * (n + 1) * s02
         self.VI_norm = v_num / v_den - (1.0 / (n - 1)) ** 2
         self.seI_norm = self.VI_norm ** (1 / 2.)
 
-        k = (1 / ((z ** 4).sum()) * (((z ** 2).sum()) ** 2))
-        vi = (1 / (((n - 1) ** 3) * s02)) * ((n * ((n * n - 3 * n + 3)
-                                                   * s1 - n * s2 + 3 * s02))
-                                             - (k * ((n * n - n) * s1 - 2 * n *
-                                                     s2 + 6 * s02)))
-        self.VI_rand = vi
-        self.seI_rand = vi ** (1 / 2.)
+        # variance under randomization
+        xd4 = z**4
+        xd2 = z**2
+        k_num = xd4.sum() / n
+        k_den = (xd2.sum() / n)**2
+        k = k_num / k_den
+        EI = self.EI
+        A = n * ((n2 - 3 * n + 3) * s1 - n * s2 + 3 * s02)
+        B = k * ((n2 - n) * s1 - 2 * n * s2 + 6 * s02  )
+        VIR = (A - B) / ((n - 1) * (n - 2) * (n - 3 ) * s02) - EI*EI
+        self.VI_rand = VIR 
+        self.seI_rand = VIR ** (1 / 2.)
 
     def __calc(self, z):
         zl = slag(self.w, z)
         inum = (z * zl).sum()
         return self.n / self.w.s0 * inum / self.z2ss
 
+    @property
+    def _statistic(self):
+        """More consistent hidden attribute to access ESDA statistics"""
+        return self.I
 
-class Moran_BV:
+    @classmethod
+    def by_col(cls, df, cols, w=None, inplace=False, pvalue='sim', outvals=None, **stat_kws):
+        """ 
+        Function to compute a Moran statistic on a dataframe
+
+        Arguments
+        ---------
+        df          :   pandas.DataFrame
+                        a pandas dataframe with a geometry column
+        cols        :   string or list of string
+                        name or list of names of columns to use to compute the statistic
+        w           :   pysal weights object
+                        a weights object aligned with the dataframe. If not provided, this
+                        is searched for in the dataframe's metadata
+        inplace     :   bool
+                        a boolean denoting whether to operate on the dataframe inplace or to
+                        return a series contaning the results of the computation. If
+                        operating inplace, the derived columns will be named
+                        'column_moran'
+        pvalue      :   string
+                        a string denoting which pvalue should be returned. Refer to the
+                        the Moran statistic's documentation for available p-values
+        outvals     :   list of strings
+                        list of arbitrary attributes to return as columns from the 
+                        Moran statistic
+        **stat_kws  :   keyword arguments
+                        options to pass to the underlying statistic. For this, see the
+                        documentation for the Moran statistic.
+
+        Returns
+        --------
+        If inplace, None, and operation is conducted on dataframe in memory. Otherwise,
+        returns a copy of the dataframe with the relevant columns attached.
+
+        See Also
+        ---------
+        For further documentation, refer to the Moran class in pysal.esda
+        """
+        return _univariate_handler(df, cols, w=w, inplace=inplace, pvalue=pvalue, 
+                                   outvals=outvals, stat=cls,
+                                   swapname=cls.__name__.lower(), **stat_kws)
+    
+class Moran_BV(object):
     """
     Bivariate Moran's I
 
@@ -304,6 +368,8 @@ class Moran_BV:
 
     """
     def __init__(self, x, y, w, transformation="r", permutations=PERMUTATIONS):
+        x = np.asarray(x).flatten()
+        y = np.asarray(y).flatten()
         zy = (y - y.mean()) / y.std(ddof=1)
         zx = (x - x.mean()) / x.std(ddof=1)
         self.zx = zx
@@ -336,6 +402,60 @@ class Moran_BV:
         self.num = (self.zx * wzy).sum()
         return self.num / self.den
 
+    @property
+    def _statistic(self):
+        """More consistent hidden attribute to access ESDA statistics"""
+        return self.I
+    
+    @classmethod
+    def by_col(cls, df, x, y=None, w=None, inplace=False, pvalue='sim', outvals=None, **stat_kws):
+        """ 
+        Function to compute a Moran_BV statistic on a dataframe
+
+        Arguments
+        ---------
+        df          :   pandas.DataFrame
+                        a pandas dataframe with a geometry column
+        X           :   list of strings
+                        column name or list of column names to use as X values to compute
+                        the bivariate statistic. If no Y is provided, pairwise comparisons
+                        among these variates are used instead. 
+        Y           :   list of strings
+                        column name or list of column names to use as Y values to compute
+                        the bivariate statistic. if no Y is provided, pariwise comparisons
+                        among the X variates are used instead. 
+        w           :   pysal weights object
+                        a weights object aligned with the dataframe. If not provided, this
+                        is searched for in the dataframe's metadata
+        inplace     :   bool
+                        a boolean denoting whether to operate on the dataframe inplace or to
+                        return a series contaning the results of the computation. If
+                        operating inplace, the derived columns will be named
+                        'column_moran_local'
+        pvalue      :   string
+                        a string denoting which pvalue should be returned. Refer to the
+                        the Moran_BV statistic's documentation for available p-values
+        outvals     :   list of strings
+                        list of arbitrary attributes to return as columns from the 
+                        Moran_BV statistic
+        **stat_kws  :   keyword arguments
+                        options to pass to the underlying statistic. For this, see the
+                        documentation for the Moran_BV statistic.
+
+
+        Returns
+        --------
+        If inplace, None, and operation is conducted on dataframe in memory. Otherwise,
+        returns a copy of the dataframe with the relevant columns attached.
+
+        See Also
+        ---------
+        For further documentation, refer to the Moran_BV class in pysal.esda
+        """
+        return _bivariate_handler(df, x, y=y, w=w, inplace=inplace, 
+                                  pvalue = pvalue, outvals = outvals, 
+                                  swapname=cls.__name__.lower(), stat=cls,**stat_kws)
+
 
 def Moran_BV_matrix(variables, w, permutations=0, varnames=None):
     """Bivariate Moran Matrix
@@ -402,7 +522,6 @@ def Moran_BV_matrix(variables, w, permutations=0, varnames=None):
             results[j, i] = Moran_BV(y2, y1, w, permutations=permutations)
     return results
 
-
 class Moran_Rate(Moran):
     """
     Adjusted Moran's I Global Autocorrelation Statistic for Rate
@@ -510,24 +629,107 @@ class Moran_Rate(Moran):
 
     def __init__(self, e, b, w, adjusted=True, transformation="r",
                  permutations=PERMUTATIONS, two_tailed=True):
+        e = np.asarray(e).flatten()
+        b = np.asarray(b).flatten()
         if adjusted:
             y = assuncao_rate(e, b)
         else:
             y = e * 1.0 / b
         Moran.__init__(self, y, w, transformation=transformation,
                        permutations=permutations, two_tailed=two_tailed)
+    
+    @classmethod
+    def by_col(cls, df, events, populations, w=None, inplace=False, 
+               pvalue='sim', outvals=None, swapname='',  **stat_kws):
+        """ 
+        Function to compute a Moran_Rate statistic on a dataframe
 
+        Arguments
+        ---------
+        df          :   pandas.DataFrame
+                        a pandas dataframe with a geometry column
+        events      :   string or  list of strings
+                        one or more names where events are stored
+        populations :   string or list of strings
+                        one or more names where the populations corresponding to the
+                        events are stored. If one population column is provided, it is
+                        used for all event columns. If more than one population column
+                        is provided but there is not a population for every event
+                        column, an exception will be raised.
+        w           :   pysal weights object
+                        a weights object aligned with the dataframe. If not provided, this
+                        is searched for in the dataframe's metadata
+        inplace     :   bool
+                        a boolean denoting whether to operate on the dataframe inplace or to
+                        return a series contaning the results of the computation. If
+                        operating inplace, the derived columns will be named
+                        'column_moran_rate'
+        pvalue      :   string
+                        a string denoting which pvalue should be returned. Refer to the
+                        the Moran_Rate statistic's documentation for available p-values
+        outvals     :   list of strings
+                        list of arbitrary attributes to return as columns from the 
+                        Moran_Rate statistic
+        **stat_kws  :   keyword arguments
+                        options to pass to the underlying statistic. For this, see the
+                        documentation for the Moran_Rate statistic.
 
-class Moran_Local:
+        Returns
+        --------
+        If inplace, None, and operation is conducted on dataframe in memory. Otherwise,
+        returns a copy of the dataframe with the relevant columns attached.
+
+        See Also
+        ---------
+        For further documentation, refer to the Moran_Rate class in pysal.esda
+        """
+        if not inplace:
+            new = df.copy()
+            cls.by_col(new, events, populations, w=w, inplace=True,
+                              pvalue=pvalue, outvals=outvals, swapname=swapname,
+                              **stat_kws)
+            return new
+        if isinstance(events, str):
+            events = [events]
+        if isinstance(populations, str): 
+            populations = [populations]
+        if len(populations) < len(events):
+            populations = populations * len(events)
+        if len(events) != len(populations):
+            raise ValueError('There is not a one-to-one matching between events and '
+                              'populations!\nEvents: {}\n\nPopulations:'
+                              ' {}'.format(events, populations))
+        adjusted = stat_kws.pop('adjusted', True)
+
+        if isinstance(adjusted, bool):
+            adjusted = [adjusted] * len(events)
+        if swapname is '':
+            swapname = cls.__name__.lower()
+
+        rates = [assuncao_rate(df[e], df[pop]) if adj
+                 else df[e].astype(float) / df[pop] 
+                 for e,pop,adj in zip(events, populations, adjusted)]
+        names = ['-'.join((e,p)) for e,p in zip(events, populations)]
+        out_df = df.copy()
+        rate_df = out_df.from_items(zip(names, rates)) #trick to avoid importing pandas
+        stat_df = _univariate_handler(rate_df, names, w=w, inplace=False, 
+                                      pvalue = pvalue, outvals = outvals, 
+                                      swapname=swapname, 
+                                      stat=Moran, #how would this get done w/super?
+                                      **stat_kws)
+        for col in stat_df.columns:
+            df[col] = stat_df[col]
+
+class Moran_Local(object):
     """Local Moran Statistics
 
 
     Parameters
     ----------
-    y : array
-        (n,1), attribute array
-    w : W
-        weight instance assumed to be aligned with y
+    y              : array
+                     (n,1), attribute array
+    w              : W
+                     weight instance assumed to be aligned with y
     transformation : {'R', 'B', 'D', 'U', 'V'}
                      weights transformation,  default is row-standardized "r".
                      Other options include
@@ -610,6 +812,7 @@ class Moran_Local:
     """
     def __init__(self, y, w, transformation="r", permutations=PERMUTATIONS,
                  geoda_quads=False):
+        y = np.asarray(y).flatten()
         self.y = y
         n = len(y)
         self.n = n
@@ -647,7 +850,7 @@ class Moran_Local:
             self.VI_sim = self.seI_sim * self.seI_sim
             self.z_sim = (self.Is - self.EI_sim) / self.seI_sim
             self.p_z_sim = 1 - stats.norm.cdf(np.abs(self.z_sim))
-
+        
     def calc(self, w, z):
         zl = slag(w, z)
         return self.n_1 * self.z * zl / self.den
@@ -695,19 +898,66 @@ class Moran_Local:
         self.q = self.quads[0] * pp + self.quads[1] * np + self.quads[2] * nn \
             + self.quads[3] * pn
 
+    @property
+    def _statistic(self):
+        """More consistent hidden attribute to access ESDA statistics"""
+        return self.Is
+    
+    @classmethod
+    def by_col(cls, df, cols, w=None, inplace=False, pvalue='sim', outvals=None, **stat_kws):
+        """ 
+        Function to compute a Moran_Local statistic on a dataframe
 
-class Moran_Local_BV:
+        Arguments
+        ---------
+        df          :   pandas.DataFrame
+                        a pandas dataframe with a geometry column
+        cols        :   string or list of string
+                        name or list of names of columns to use to compute the statistic
+        w           :   pysal weights object
+                        a weights object aligned with the dataframe. If not provided, this
+                        is searched for in the dataframe's metadata
+        inplace     :   bool
+                        a boolean denoting whether to operate on the dataframe inplace or to
+                        return a series contaning the results of the computation. If
+                        operating inplace, the derived columns will be named
+                        'column_moran_local'
+        pvalue      :   string
+                        a string denoting which pvalue should be returned. Refer to the
+                        the Moran_Local statistic's documentation for available p-values
+        outvals     :   list of strings
+                        list of arbitrary attributes to return as columns from the 
+                        Moran_Local statistic
+        **stat_kws  :   keyword arguments
+                        options to pass to the underlying statistic. For this, see the
+                        documentation for the Moran_Local statistic.
+
+        Returns
+        --------
+        If inplace, None, and operation is conducted on dataframe in memory. Otherwise,
+        returns a copy of the dataframe with the relevant columns attached.
+
+        See Also
+        ---------
+        For further documentation, refer to the Moran_Local class in pysal.esda
+        """
+        return _univariate_handler(df, cols, w=w, inplace=inplace, pvalue=pvalue, 
+                                   outvals=outvals, stat=cls,
+                                   swapname=cls.__name__.lower(), **stat_kws)
+
+
+class Moran_Local_BV(object):
     """Bivariate Local Moran Statistics
 
 
     Parameters
     ----------
-    x : array
-        x-axis variable
-    y : array
-        (n,1), wy will be on y axis
-    w : W
-        weight instance assumed to be aligned with y
+    x              : array
+                     x-axis variable
+    y              : array
+                     (n,1), wy will be on y axis
+    w              : W
+                     weight instance assumed to be aligned with y
     transformation : {'R', 'B', 'D', 'U', 'V'}
                      weights transformation,  default is row-standardized "r".
                      Other options include
@@ -795,6 +1045,8 @@ class Moran_Local_BV:
     """
     def __init__(self, x, y, w, transformation="r", permutations=PERMUTATIONS,
                  geoda_quads=False):
+        x = np.asarray(x).flatten()
+        y = np.asarray(y).flatten()
         self.y = y
         n = len(y)
         self.n = n
@@ -885,6 +1137,59 @@ class Moran_Local_BV:
         self.q = self.quads[0] * pp + self.quads[1] * np + self.quads[2] * nn \
             + self.quads[3] * pn
 
+    @property
+    def _statistic(self):
+        """More consistent hidden attribute to access ESDA statistics"""
+        return self.Is
+
+    @classmethod
+    def by_col(cls, df, x, y=None, w=None, inplace=False, pvalue='sim', outvals=None, **stat_kws):
+        """ 
+        Function to compute a Moran_Local_BV statistic on a dataframe
+
+        Arguments
+        ---------
+        df          :   pandas.DataFrame
+                        a pandas dataframe with a geometry column
+        X           :   list of strings
+                        column name or list of column names to use as X values to compute
+                        the bivariate statistic. If no Y is provided, pairwise comparisons
+                        among these variates are used instead. 
+        Y           :   list of strings
+                        column name or list of column names to use as Y values to compute
+                        the bivariate statistic. if no Y is provided, pariwise comparisons
+                        among the X variates are used instead. 
+        w           :   pysal weights object
+                        a weights object aligned with the dataframe. If not provided, this
+                        is searched for in the dataframe's metadata
+        inplace     :   bool
+                        a boolean denoting whether to operate on the dataframe inplace or to
+                        return a series contaning the results of the computation. If
+                        operating inplace, the derived columns will be named
+                        'column_moran_local_bv'
+        pvalue      :   string
+                        a string denoting which pvalue should be returned. Refer to the
+                        the Moran_Local_BV statistic's documentation for available p-values
+        outvals     :   list of strings
+                        list of arbitrary attributes to return as columns from the 
+                        Moran_Local_BV statistic
+        **stat_kws  :   keyword arguments
+                        options to pass to the underlying statistic. For this, see the
+                        documentation for the Moran_Local_BV statistic.
+
+
+        Returns
+        --------
+        If inplace, None, and operation is conducted on dataframe in memory. Otherwise,
+        returns a copy of the dataframe with the relevant columns attached.
+
+        See Also
+        ---------
+        For further documentation, refer to the Moran_Local_BV class in pysal.esda
+        """
+        return _bivariate_handler(df, x, y=y, w=w, inplace=inplace, 
+                                  pvalue = pvalue, outvals = outvals, 
+                                  swapname=cls.__name__.lower(), stat=cls,**stat_kws)
 
 class Moran_Local_Rate(Moran_Local):
     """
@@ -892,15 +1197,15 @@ class Moran_Local_Rate(Moran_Local):
 
     Parameters
     ----------
-    e : array
-        (n,1), an event variable across n spatial units
-    b : array
-        (n,1), a population-at-risk variable across n spatial units
-    w : W
-        weight instance assumed to be aligned with y
-    adjusted : boolean
-              whether or not local Moran statistics need to be adjusted for
-              rate variable
+    e              : array
+                     (n,1), an event variable across n spatial units
+    b              : array
+                     (n,1), a population-at-risk variable across n spatial units
+    w              : W
+                     weight instance assumed to be aligned with y
+    adjusted       : boolean
+                     whether or not local Moran statistics need to be adjusted for
+                     rate variable
     transformation : {'R', 'B', 'D', 'U', 'V'}
                      weights transformation,  default is row-standardized "r".
                      Other options include
@@ -917,47 +1222,47 @@ class Moran_Local_Rate(Moran_Local):
                      If False use PySAL Scheme: HH=1, LH=2, LL=3, HL=4
     Attributes
     ----------
-    y            : array
-                   rate variables computed from parameters e and b
-                   if adjusted is True, y is standardized rates
-                   otherwise, y is raw rates
-    w            : W
-                   original w object
-    permutations : int
-                   number of random permutations for calculation of pseudo
-                   p_values
-    I            : float
-                   value of Moran's I
-    q            : array
-                   (if permutations>0)
-                   values indicate quandrant location 1 HH,  2 LH,  3 LL,  4 HL
-    sim          : array
-                   (if permutations>0)
-                   vector of I values for permuted samples
-    p_sim        : array
-                   (if permutations>0)
-                   p-value based on permutations (one-sided)
-                   null: spatial randomness
-                   alternative: the observed Ii is further away or extreme
-                   from the median of simulated Iis. It is either extremely
-                   high or extremely low in the distribution of simulated Is
-    EI_sim       : float
-                   (if permutations>0)
-                   average value of I from permutations
-    VI_sim       : float
-                   (if permutations>0)
-                   variance of I from permutations
-    seI_sim      : float
-                   (if permutations>0)
-                   standard deviation of I under permutations.
-    z_sim        : float
-                   (if permutations>0)
-                   standardized I based on permutations
-    p_z_sim      : float
-                   (if permutations>0)
-                   p-value based on standard normal approximation from
-                   permutations (one-sided)
-                   for two-sided tests, these values should be multiplied by 2
+    y              : array
+                     rate variables computed from parameters e and b
+                     if adjusted is True, y is standardized rates
+                     otherwise, y is raw rates
+    w              : W
+                     original w object
+    permutations   : int
+                     number of random permutations for calculation of pseudo
+                     p_values
+    Is             : float
+                     local Moran's I values for rates or Empirical Bayes Index
+    q              : array
+                     (if permutations>0)
+                     values indicate quandrant location 1 HH,  2 LH,  3 LL,  4 HL
+    sim            : array
+                     (if permutations>0)
+                     vector of I values for permuted samples
+    p_sim          : array
+                     (if permutations>0)
+                     p-values based on permutations (one-sided)
+                     null: spatial randomness
+                     alternative: the observed Ii is further away or extreme
+                     from the median of simulated values. It is either extremelyi
+                     high or extremely low in the distribution of simulated Is.
+    EI_sim         : array
+                     (if permutations>0)
+                     average values of local Is from permutations
+    VI_sim         : array
+                     (if permutations>0)
+                     variance of Is from permutations
+    seI_sim        : array
+                     (if permutations>0)
+                     standard deviations of Is under permutations.
+    z_sim          : arrray
+                     (if permutations>0)
+                     standardized Is based on permutations
+    p_z_sim        : array
+                     (if permutations>0)
+                     p-values based on standard normal approximation from
+                     permutations (one-sided)
+                     for two-sided tests, these values should be multiplied by 2
 
     Examples
     --------
@@ -989,6 +1294,8 @@ class Moran_Local_Rate(Moran_Local):
 
     def __init__(self, e, b, w, adjusted=True, transformation="r",
                  permutations=PERMUTATIONS, geoda_quads=False):
+        e = np.asarray(e).flatten()
+        b = np.asarray(b).flatten()
         if adjusted:
             y = assuncao_rate(e, b)
         else:
@@ -997,3 +1304,84 @@ class Moran_Local_Rate(Moran_Local):
                              transformation=transformation,
                              permutations=permutations,
                              geoda_quads=geoda_quads)
+    
+    @classmethod
+    def by_col(cls, df, events, populations, w=None, inplace=False, 
+               pvalue='sim', outvals=None, swapname='',  **stat_kws):
+        """ 
+        Function to compute a Moran_Local_Rate statistic on a dataframe
+
+        Arguments
+        ---------
+        df          :   pandas.DataFrame
+                        a pandas dataframe with a geometry column
+        events      :   string or  list of strings
+                        one or more names where events are stored
+        populations :   string or list of strings
+                        one or more names where the populations corresponding to the
+                        events are stored. If one population column is provided, it is
+                        used for all event columns. If more than one population column
+                        is provided but there is not a population for every event
+                        column, an exception will be raised.
+        w           :   pysal weights object
+                        a weights object aligned with the dataframe. If not provided, this
+                        is searched for in the dataframe's metadata
+        inplace     :   bool
+                        a boolean denoting whether to operate on the dataframe inplace or to
+                        return a series contaning the results of the computation. If
+                        operating inplace, the derived columns will be named 'column_moran_local_rate'
+        pvalue      :   string
+                        a string denoting which pvalue should be returned. Refer to the
+                        the Moran_Local_Rate statistic's documentation for available p-values
+        outvals     :   list of strings
+                        list of arbitrary attributes to return as columns from the 
+                        Moran_Local_Rate statistic
+        **stat_kws  :   keyword arguments
+                        options to pass to the underlying statistic. For this, see the
+                        documentation for the Moran_Local_Rate statistic.
+
+        Returns
+        --------
+        If inplace, None, and operation is conducted on dataframe in memory. Otherwise,
+        returns a copy of the dataframe with the relevant columns attached.
+
+        See Also
+        ---------
+        For further documentation, refer to the Moran_Local_Rate class in pysal.esda
+        """
+        if not inplace:
+            new = df.copy()
+            cls.by_col(new, events, populations, w=w, inplace=True,
+                              pvalue=pvalue, outvals=outvals, swapname=swapname,
+                              **stat_kws)
+            return new
+        if isinstance(events, str):
+            events = [events]
+        if isinstance(populations, str): 
+            populations = [populations]
+        if len(populations) < len(events):
+            populations = populations * len(events)
+        if len(events) != len(populations):
+            raise ValueError('There is not a one-to-one matching between events and '
+                              'populations!\nEvents: {}\n\nPopulations:'
+                              ' {}'.format(events, populations))
+        adjusted = stat_kws.pop('adjusted', True)
+
+        if isinstance(adjusted, bool):
+            adjusted = [adjusted] * len(events)
+        if swapname is '':
+            swapname = cls.__name__.lower()
+
+        rates = [assuncao_rate(df[e], df[pop]) if adj
+                 else df[e].astype(float) / df[pop] 
+                 for e,pop,adj in zip(events, populations, adjusted)]
+        names = ['-'.join((e,p)) for e,p in zip(events, populations)]
+        out_df = df.copy()
+        rate_df = out_df.from_items(zip(names, rates)) #trick to avoid importing pandas
+        _univariate_handler(rate_df, names, w=w, inplace=True, 
+                                      pvalue = pvalue, outvals = outvals, 
+                                      swapname=swapname,
+                                      stat=Moran_Local, #how would this get done w/super?
+                                      **stat_kws)
+        for col in rate_df.columns:
+            df[col] = rate_df[col]
