@@ -1,13 +1,16 @@
 import unittest
 import numpy as np
-import pysal
-from pysal.esda.join_counts import Join_Counts
 
+from ..join_counts import Join_Counts
+from ...weights import lat2W
+from ...common import pandas
+
+PANDAS_EXTINCT = pandas is None
 
 class Join_Counts_Tester(unittest.TestCase):
     """Unit test for Join Counts"""
     def setUp(self):
-        self.w = pysal.lat2W(4, 4)
+        self.w = lat2W(4, 4)
         self.y = np.ones(16)
         self.y[0:8] = 0
 
@@ -29,6 +32,24 @@ class Join_Counts_Tester(unittest.TestCase):
         self.assertAlmostEquals(np.mean(jc.sim_bw), 12.811811811811811)
         self.assertAlmostEquals(np.max(jc.sim_bw), 24.0)
         self.assertAlmostEquals(np.min(jc.sim_bw), 7.0)
+    
+    @unittest.skipIf(PANDAS_EXTINCT, 'missing pandas')
+    def test_by_col(self):
+        import pandas as pd
+        df = pd.DataFrame(self.y, columns=['y'])
+        np.random.seed(12345)
+        r1 = Join_Counts.by_col(df, ['y'], w=self.w, permutations=999)
+        
+        bb = np.unique(r1.y_bb.values)
+        bw = np.unique(r1.y_bw.values)
+        bb_p = np.unique(r1.y_p_sim_bb.values)
+        bw_p = np.unique(r1.y_p_sim_bw.values)
+        np.random.seed(12345)
+        c = Join_Counts(self.y, self.w, permutations=999)
+        self.assertAlmostEquals(bb, c.bb)
+        self.assertAlmostEquals(bw, c.bw)
+        self.assertAlmostEquals(bb_p, c.p_sim_bb)
+        self.assertAlmostEquals(bw_p, c.p_sim_bw)
 
 suite = unittest.TestSuite()
 test_classes = [Join_Counts_Tester]
